@@ -1,4 +1,4 @@
-# -*- coding: utf-8 -*-
+# coding: utf-8
 """
 Integration Tests
 ~~~~~~~~~~~~~~~~~
@@ -7,30 +7,30 @@ import sys
 import re
 import argparse
 
+import iocapture
+try:
+    import unittest.mock as mock
+except ImportError:
+    # FIXME: remove in v.0.28
+    import mock
 import pytest
 
 import argh
+from argh.exceptions import AssemblingError
 
-from .base import DebugArghParser, run
-
-
-@pytest.mark.xfail(reason='TODO')
-def test_guessing_integration():
-    "guessing is used in dispatching"
-    pass
+from .base import DebugArghParser, get_usage_string, run, CmdResult as R
 
 
 def test_set_default_command_integration():
-    @argh.arg('--foo', default=1)
-    def cmd(args):
-        return args.foo
+    def cmd(foo=1):
+        return foo
 
     p = DebugArghParser()
     p.set_default_command(cmd)
 
-    assert run(p, '') == '1\n'
-    assert run(p, '--foo 2') == '2\n'
-    assert None == run(p, '--help', exit=True)
+    assert run(p, '') == R(out='1\n', err='')
+    assert run(p, '--foo 2') == R(out='2\n', err='')
+    assert run(p, '--help', exit=True) == None
 
 
 def test_set_default_command_integration_merging():
@@ -41,8 +41,8 @@ def test_set_default_command_integration_merging():
     p = DebugArghParser()
     p.set_default_command(cmd)
 
-    assert run(p, '') == '1\n'
-    assert run(p, '--foo 2') == '2\n'
+    assert run(p, '') == R(out='1\n', err='')
+    assert run(p, '--foo 2') == R(out='2\n', err='')
     assert 'bar' in p.format_help()
 
 
@@ -58,7 +58,7 @@ def test_simple_function_no_args():
     p = DebugArghParser()
     p.set_default_command(cmd)
 
-    assert run(p, '') == '1\n'
+    assert run(p, '') == R(out='1\n', err='')
 
 
 def test_simple_function_positional():
@@ -73,7 +73,7 @@ def test_simple_function_positional():
     else:
         msg = 'the following arguments are required: x'
     assert run(p, '', exit=True) == msg
-    assert run(p, 'foo') == 'foo\n'
+    assert run(p, 'foo') == R(out='foo\n', err='')
 
 
 def test_simple_function_defaults():
@@ -83,9 +83,9 @@ def test_simple_function_defaults():
     p = DebugArghParser()
     p.set_default_command(cmd)
 
-    assert run(p, '') == 'foo\n'
+    assert run(p, '') == R(out='foo\n', err='')
     assert run(p, 'bar', exit=True) == 'unrecognized arguments: bar'
-    assert run(p, '--x bar') == 'bar\n'
+    assert run(p, '--x bar') == R(out='bar\n', err='')
 
 
 def test_simple_function_varargs():
@@ -97,9 +97,9 @@ def test_simple_function_varargs():
     p = DebugArghParser()
     p.set_default_command(func)
 
-    assert run(p, '') == '\n'
-    assert run(p, 'foo') == 'foo\n'
-    assert run(p, 'foo bar') == 'foo, bar\n'
+    assert run(p, '') == R(out='\n', err='')
+    assert run(p, 'foo') == R(out='foo\n', err='')
+    assert run(p, 'foo bar') == R(out='foo, bar\n', err='')
 
 
 def test_simple_function_kwargs():
@@ -120,21 +120,24 @@ def test_simple_function_kwargs():
     else:
         msg = 'the following arguments are required: foo'
     assert run(p, '', exit=True) == msg
-    assert run(p, 'hello') == 'bar: None\nfoo: hello\n'
+    assert run(p, 'hello') == R(out='bar: None\nfoo: hello\n', err='')
     assert run(p, '--bar 123', exit=True) == msg
-    assert run(p, 'hello --bar 123') == 'bar: 123\nfoo: hello\n'
+    assert run(p, 'hello --bar 123') == R(out='bar: 123\nfoo: hello\n', err='')
 
 
+@pytest.mark.xfail
 def test_simple_function_multiple():
-    pass
+    raise NotImplementedError
 
 
+@pytest.mark.xfail
 def test_simple_function_nested():
-    pass
+    raise NotImplementedError
 
 
+@pytest.mark.xfail
 def test_class_method_as_command():
-    pass
+    raise NotImplementedError
 
 
 def test_all_specs_in_one():
@@ -157,30 +160,30 @@ def test_all_specs_in_one():
     #    will still have higher priority than bar.
     # 2) *args, a positional with nargs='*', sits between two required
     #    positionals (foo and fox), so it gets nothing.
-    assert run(p, 'one two') == (
+    assert run(p, 'one two') == R(out=
         'foo: one\n'
         'bar: 1\n'
         '*args: ()\n'
         '** baz: None\n'
-        '** fox: two\n')
+        '** fox: two\n', err='')
 
     # two required positionals (foo and fox) get an argument each and one extra
     # is left; therefore the middle one is given to *args.
-    assert run(p, 'one two three') == (
+    assert run(p, 'one two three') == R(out=
         'foo: one\n'
         'bar: 1\n'
         "*args: ('two',)\n"
         '** baz: None\n'
-        '** fox: three\n')
+        '** fox: three\n', err='')
 
     # two required positionals (foo and fox) get an argument each and two extra
     # are left; both are given to *args (it's greedy).
-    assert run(p, 'one two three four') == (
+    assert run(p, 'one two three four') == R(out=
         'foo: one\n'
         'bar: 1\n'
         "*args: ('two', 'three')\n"
         '** baz: None\n'
-        '** fox: four\n')
+        '** fox: four\n', err='')
 
 
 def test_arg_merged():
@@ -200,14 +203,14 @@ def test_arg_merged():
 
 
 def test_arg_mismatch_positional():
-    """ @arg must match function signature if @command is applied.
+    """ An `@arg('positional')` must match function signature.
     """
     @argh.arg('bogus-argument')
     def confuse_a_cat(vet, funny_things=123):
         return vet, funny_things
 
     p = DebugArghParser('PROG')
-    with pytest.raises(ValueError) as excinfo:
+    with pytest.raises(AssemblingError) as excinfo:
         p.set_default_command(confuse_a_cat)
 
     msg = ("confuse_a_cat: argument bogus-argument does not fit "
@@ -216,14 +219,14 @@ def test_arg_mismatch_positional():
 
 
 def test_arg_mismatch_flag():
-    """ @arg must match function signature if @command is applied.
+    """ An `@arg('--flag')` must match function signature.
     """
     @argh.arg('--bogus-argument')
     def confuse_a_cat(vet, funny_things=123):
         return vet, funny_things
 
     p = DebugArghParser('PROG')
-    with pytest.raises(ValueError) as excinfo:
+    with pytest.raises(AssemblingError) as excinfo:
         p.set_default_command(confuse_a_cat)
 
     msg = ("confuse_a_cat: argument --bogus-argument does not fit "
@@ -231,38 +234,43 @@ def test_arg_mismatch_flag():
     assert msg in str(excinfo.value)
 
 
-def test_backwards_compatibility_issue29():
+def test_arg_mismatch_positional_vs_flag():
+    """ An `@arg('arg')` must match a positional arg in function signature.
+    """
     @argh.arg('foo')
-    @argh.arg('--bar', default=1)
-    def old(args):
-        yield '{0} {1}'.format(args.foo, args.bar)
-
-    @argh.command
-    def old_marked(foo, bar=1):
-        yield '{0} {1}'.format(foo, bar)
-
-    def new(foo, bar=1):
-        yield '{0} {1}'.format(foo, bar)
+    def func(foo=123):
+        return foo
 
     p = DebugArghParser('PROG')
-    p.add_commands([old, old_marked, new])
+    with pytest.raises(AssemblingError) as excinfo:
+        p.set_default_command(func)
 
-    assert 'ok 1\n' == run(p, 'old ok')
-    assert 'ok 5\n' == run(p, 'old ok --bar 5')
+    msg = ('func: argument "foo" declared as optional (in function signature)'
+           ' and positional (via decorator)')
+    assert msg in str(excinfo.value)
 
-    assert 'ok 1\n' == run(p, 'old-marked ok')
-    assert 'ok 5\n' == run(p, 'old-marked ok --bar 5')
 
-    assert 'ok 1\n' == run(p, 'new ok')
-    assert 'ok 5\n' == run(p, 'new ok --bar 5')
+def test_arg_mismatch_flag_vs_positional():
+    """ An `@arg('--flag')` must match a keyword in function signature.
+    """
+    @argh.arg('--foo')
+    def func(foo):
+        return foo
+
+    p = DebugArghParser('PROG')
+    with pytest.raises(AssemblingError) as excinfo:
+        p.set_default_command(func)
+
+    msg = ('func: argument "foo" declared as positional (in function signature)'
+           ' and optional (via decorator)')
+    assert msg in str(excinfo.value)
 
 
 class TestErrorWrapping:
 
     def _get_parrot(self):
-        @argh.arg('--dead', default=False)
-        def parrot(args):
-            if args.dead:
+        def parrot(dead=False):
+            if dead:
                 raise ValueError('this parrot is no more')
             else:
                 return 'beautiful plumage'
@@ -275,27 +283,52 @@ class TestErrorWrapping:
         p = DebugArghParser()
         p.set_default_command(parrot)
 
-        assert run(p, '') == 'beautiful plumage\n'
+        assert run(p, '') == R('beautiful plumage\n', '')
         with pytest.raises(ValueError) as excinfo:
             run(p, '--dead')
         assert re.match('this parrot is no more', str(excinfo.value))
 
     def test_error_wrapped(self):
         parrot = self._get_parrot()
-        wrapped_parrot = argh.wrap_errors(ValueError)(parrot)
+        wrapped_parrot = argh.wrap_errors([ValueError])(parrot)
 
         p = DebugArghParser()
         p.set_default_command(wrapped_parrot)
 
-        assert run(p, '') == 'beautiful plumage\n'
-        assert run(p, '--dead') == 'this parrot is no more\n'
+        assert run(p, '') == R('beautiful plumage\n', '')
+        assert run(p, '--dead') == R('', 'ValueError: this parrot is no more\n')
+
+    def test_processor(self):
+        parrot = self._get_parrot()
+        wrapped_parrot = argh.wrap_errors([ValueError])(parrot)
+
+        def failure(err):
+            return 'ERR: ' + str(err) + '!'
+        processed_parrot = argh.wrap_errors(processor=failure)(wrapped_parrot)
+
+        p = argh.ArghParser()
+        p.set_default_command(processed_parrot)
+
+        assert run(p, '--dead') == R('', 'ERR: this parrot is no more!\n')
+
+    def test_stderr_vs_stdout(self):
+
+        @argh.wrap_errors([KeyError])
+        def func(key):
+            db = {'a': 1}
+            return db[key]
+
+        p = argh.ArghParser()
+        p.set_default_command(func)
+
+        assert run(p, 'a') == R(out='1\n', err='')
+        assert run(p, 'b') == R(out='', err="KeyError: 'b'\n")
 
 
 def test_argv():
 
-    @argh.arg('text')
-    def echo(args):
-        return 'you said {0}'.format(args.text)
+    def echo(text):
+        return 'you said {0}'.format(text)
 
     p = DebugArghParser()
     p.add_commands([echo])
@@ -303,7 +336,7 @@ def test_argv():
     _argv = sys.argv
 
     sys.argv = sys.argv[:1] + ['echo', 'hi there']
-    assert run(p, None) == 'you said hi there\n'
+    assert run(p, None) == R('you said hi there\n', '')
 
     sys.argv = _argv
 
@@ -311,8 +344,8 @@ def test_argv():
 def test_commands_not_defined():
     p = DebugArghParser()
 
-    assert run(p, '', {'raw_output': True}) == p.format_usage()
-    assert run(p, '') == p.format_usage() + '\n'
+    assert run(p, '', {'raw_output': True}).out == p.format_usage()
+    assert run(p, '').out == p.format_usage() + '\n'
 
     assert 'unrecognized arguments' in run(p, 'foo', exit=True)
     assert 'unrecognized arguments' in run(p, '--foo', exit=True)
@@ -330,7 +363,7 @@ def test_command_not_chosen():
         assert 'too few arguments' in run(p, '', exit=True)
     else:
         # Python since 3.3 returns a help message and doesn't exit
-        assert 'usage:' in run(p, '')
+        assert 'usage:' in run(p, '').out
 
 
 def test_invalid_choice():
@@ -342,7 +375,7 @@ def test_invalid_choice():
     p = DebugArghParser()
     p.add_commands([cmd])
 
-    assert run(p, 'bar', exit=True).startswith('invalid choice')
+    assert 'invalid choice' in run(p, 'bar', exit=True)
 
     if sys.version_info < (3,3):
         # Python before 3.3 exits with a less informative error
@@ -356,7 +389,7 @@ def test_invalid_choice():
     p = DebugArghParser()
     p.add_commands([cmd], namespace='nest')
 
-    assert run(p, 'nest bar', exit=True).startswith('invalid choice')
+    assert 'invalid choice' in run(p, 'nest bar', exit=True)
 
     if sys.version_info < (3,3):
         # Python before 3.3 exits with a less informative error
@@ -367,7 +400,7 @@ def test_invalid_choice():
 
 
 def test_unrecognized_arguments():
-    def cmd(args):
+    def cmd():
         return 1
 
     # single-command parser
@@ -390,28 +423,26 @@ def test_unrecognized_arguments():
 def test_echo():
     "A simple command is resolved to a function."
 
-    @argh.arg('text')
-    def echo(args):
-        return 'you said {0}'.format(args.text)
+    def echo(text):
+        return 'you said {0}'.format(text)
 
     p = DebugArghParser()
     p.add_commands([echo])
 
-    assert run(p, 'echo foo') == 'you said foo\n'
+    assert run(p, 'echo foo') == R(out='you said foo\n', err='')
 
 
 def test_bool_action():
     "Action `store_true`/`store_false` is inferred from default value."
 
-    @argh.arg('--dead', default=False)
-    def parrot(args):
-        return 'this parrot is no more' if args.dead else 'beautiful plumage'
+    def parrot(dead=False):
+        return 'this parrot is no more' if dead else 'beautiful plumage'
 
     p = DebugArghParser()
     p.add_commands([parrot])
 
-    assert run(p, 'parrot') == 'beautiful plumage\n'
-    assert run(p, 'parrot --dead') == 'this parrot is no more\n'
+    assert run(p, 'parrot').out == 'beautiful plumage\n'
+    assert run(p, 'parrot --dead').out == 'this parrot is no more\n'
 
 
 def test_bare_namespace():
@@ -430,7 +461,7 @@ def test_bare_namespace():
         assert run(p, 'greet', exit=True) == 'too few arguments'
     else:
         # Python since 3.3 returns a help message and doesn't exit
-        assert 'usage:' in run(p, 'greet', exit=True)
+        assert 'usage:' in run(p, 'greet', exit=True).out
 
     # with an argument
 
@@ -455,8 +486,8 @@ def test_namespaced_function():
     p = DebugArghParser()
     p.add_commands([hello, howdy], namespace='greet')
 
-    assert run(p, 'greet hello') == 'Hello world!\n'
-    assert run(p, 'greet hello --name=John') == 'Hello John!\n'
+    assert run(p, 'greet hello').out == 'Hello world!\n'
+    assert run(p, 'greet hello --name=John').out == 'Hello John!\n'
     assert run(p, 'greet hello John', exit=True) == 'unrecognized arguments: John'
 
     if sys.version_info < (3,3):
@@ -467,7 +498,7 @@ def test_namespaced_function():
         message = 'the following arguments are required: buddy'
 
     assert message in run(p, 'greet howdy --name=John', exit=True)
-    assert run(p, 'greet howdy John') == 'Howdy John?\n'
+    assert run(p, 'greet howdy John').out == 'Howdy John?\n'
 
 
 def test_explicit_cmd_name():
@@ -478,8 +509,8 @@ def test_explicit_cmd_name():
 
     p = DebugArghParser()
     p.add_commands([orig_name])
-    assert run(p, 'orig-name', exit=True).startswith('invalid choice')
-    assert run(p, 'new-name') == 'ok\n'
+    assert 'invalid choice' in run(p, 'orig-name', exit=True)
+    assert run(p, 'new-name').out == 'ok\n'
 
 
 def test_aliases():
@@ -492,9 +523,9 @@ def test_aliases():
     p.add_commands([alias1])
 
     if argh.assembling.SUPPORTS_ALIASES:
-        assert run(p, 'alias1') == 'ok\n'
-        assert run(p, 'alias2') == 'ok\n'
-        assert run(p, 'alias3') == 'ok\n'
+        assert run(p, 'alias1').out == 'ok\n'
+        assert run(p, 'alias2').out == 'ok\n'
+        assert run(p, 'alias3').out == 'ok\n'
 
 
 def test_help_alias():
@@ -515,14 +546,12 @@ def test_arg_order():
     """Positional arguments are resolved in the order in which the @arg
     decorators are defined.
     """
-    @argh.arg('foo')
-    @argh.arg('bar')
-    def cmd(args):
-        return args.foo, args.bar
+    def cmd(foo, bar):
+        return foo, bar
 
     p = DebugArghParser()
     p.set_default_command(cmd)
-    assert run(p, 'foo bar') == 'foo\nbar\n'
+    assert run(p, 'foo bar').out == 'foo\nbar\n'
 
 
 def test_raw_output():
@@ -534,8 +563,8 @@ def test_raw_output():
     p = DebugArghParser()
     p.set_default_command(cmd)
 
-    assert run(p, 'foo bar') == 'foo\nbar\n'
-    assert run(p, 'foo bar', {'raw_output': True}) == 'foobar'
+    assert run(p, 'foo bar').out == 'foo\nbar\n'
+    assert run(p, 'foo bar', {'raw_output': True}).out == 'foobar'
 
 
 def test_output_file():
@@ -546,8 +575,8 @@ def test_output_file():
     p = DebugArghParser()
     p.set_default_command(cmd)
 
-    assert run(p, '') == 'Hello world!\n'
-    assert run(p, '', {'output_file': None}) == 'Hello world!\n'
+    assert run(p, '').out == 'Hello world!\n'
+    assert run(p, '', {'output_file': None}).out == 'Hello world!\n'
 
 
 def test_command_error():
@@ -562,8 +591,10 @@ def test_command_error():
     p = DebugArghParser()
     p.add_commands([whiner_plain, whiner_iterable])
 
-    assert run(p, 'whiner-plain') == 'I feel depressed.\n'
-    assert run(p, 'whiner-iterable') == 'Hello...\nI feel depressed.\n'
+    assert run(p, 'whiner-plain') == R(
+        out='', err='CommandError: I feel depressed.\n')
+    assert run(p, 'whiner-iterable') == R(
+        out='Hello...\n', err='CommandError: I feel depressed.\n')
 
 
 def test_custom_namespace():
@@ -577,7 +608,7 @@ def test_custom_namespace():
     namespace = argparse.Namespace()
     namespace.custom_value = 'foo'
 
-    assert run(p, '', {'namespace': namespace}) == 'foo\n'
+    assert run(p, '', {'namespace': namespace}).out == 'foo\n'
 
 
 def test_normalized_keys():
@@ -589,4 +620,172 @@ def test_normalized_keys():
     p = DebugArghParser()
     p.set_default_command(cmd)
 
-    assert run(p, 'hello') == 'hello\n'
+    assert run(p, 'hello').out == 'hello\n'
+
+
+@mock.patch('argh.assembling.COMPLETION_ENABLED', True)
+def test_custom_argument_completer():
+    "Issue #33: Enable custom per-argument shell completion"
+
+    @argh.arg('foo', completer='STUB')
+    def func(foo):
+        pass
+
+    p = argh.ArghParser()
+    p.set_default_command(func)
+
+    assert p._actions[-1].completer == 'STUB'
+
+
+def test_class_members():
+    "Issue #34: class members as commands"
+
+    class Controller:
+        var = 123
+
+        def instance_meth(self, value):
+            return value, self.var
+
+        @classmethod
+        def class_meth(cls, value):
+            return value, cls.var
+
+        @staticmethod
+        def static_meth(value):
+            return value, 'w00t?'
+
+        @staticmethod
+        def static_meth2(value):
+            return value, 'huh!'
+
+    controller = Controller()
+
+    p = DebugArghParser()
+    p.add_commands([
+        controller.instance_meth,
+        controller.class_meth,
+        controller.static_meth,
+        Controller.static_meth2,
+    ])
+
+    assert run(p, 'instance-meth foo').out == 'foo\n123\n'
+    assert run(p, 'class-meth foo').out == 'foo\n123\n'
+    assert run(p, 'static-meth foo').out == 'foo\nw00t?\n'
+    assert run(p, 'static-meth2 foo').out == 'foo\nhuh!\n'
+
+
+def test_kwonlyargs():
+    "Correct dispatch in presence of keyword-only arguments"
+    if sys.version_info < (3,0):
+        pytest.skip('unsupported configuration')
+
+    ns = {}
+
+    exec("""def cmd(*args, foo='1', bar, baz='3', **kwargs):
+                return ' '.join(args), foo, bar, baz, len(kwargs)
+         """, None, ns)
+    cmd = ns['cmd']
+
+    p = DebugArghParser()
+    p.set_default_command(cmd)
+
+    assert (run(p, '--baz=done test  this --bar=do').out ==
+            'test this\n1\ndo\ndone\n0\n')
+    if sys.version_info < (3,3):
+        message = 'argument --bar is required'
+    else:
+        message = 'the following arguments are required: --bar'
+    assert run(p, 'test --foo=do', exit=True) == message
+
+
+def test_default_arg_values_in_help():
+    "Argument defaults should appear in the help message implicitly"
+
+    @argh.arg('name', default='Basil')
+    @argh.arg('--task', default='hang the Moose')
+    @argh.arg('--note', help='why is it a remarkable animal?')
+    def remind(name, task=None, reason='there are creatures living in it',
+               note='it can speak English'):
+        return "Oh what is it now, can't you leave me in peace..."
+
+    p = DebugArghParser()
+    p.set_default_command(remind)
+
+    assert 'Basil' in p.format_help()
+    assert 'Moose' in p.format_help()
+    assert 'creatures' in p.format_help()
+
+    # explicit help message is not obscured by the implicit one...
+    assert 'remarkable animal' in p.format_help()
+    # ...but is still present
+    assert 'it can speak' in p.format_help()
+
+
+def test_default_arg_values_in_help__regression():
+    "Empty string as default value → empty help string → broken argparse"
+
+    def foo(bar=''):
+        return bar
+
+    p = DebugArghParser()
+    p.set_default_command(foo)
+
+    # doesn't break
+    p.format_help()
+
+    # now check details
+    assert "-b BAR, --bar BAR  ''" in p.format_help()
+    # note the empty str repr ^^^
+
+
+def test_help_formatting_is_preserved():
+    "Formatting of docstrings should not be messed up in help messages"
+
+    def func():
+        """
+        Sample function.
+
+        Parameters:
+            foo: float
+                An example argument.
+            bar: bool
+                Another argument.
+        """
+        return 'hello'
+
+    p = DebugArghParser()
+    p.set_default_command(func)
+
+    assert func.__doc__ in p.format_help()
+
+
+def test_prog():
+    "Program name propagates from sys.argv[0]"
+
+    def cmd(foo=1):
+        return foo
+
+    p = DebugArghParser()
+    p.add_commands([cmd])
+
+    usage = get_usage_string()
+
+    with iocapture.capture() as captured:
+        assert run(p, '-h', exit=True) == None
+        assert captured.stdout.startswith(usage)
+
+
+def test_unknown_args():
+
+    def cmd(foo=1):
+        return foo
+
+    p = DebugArghParser()
+    p.set_default_command(cmd)
+
+    get_usage_string('[-f FOO]')
+
+    assert run(p, '--foo 1') == R(out='1\n', err='')
+    assert run(p, '--bar 1', exit=True) == 'unrecognized arguments: --bar 1'
+    assert run(p, '--bar 1', exit=False,
+               kwargs={'skip_unknown_args': True}) == R(out='1\n', err='')
